"""Identity is a module providing Identity class."""

import uuid


class Identity:
    """Identity is a class representing a unique identifier for objects.

    Identities are unmutable objects

    """

    def __init__(self, ident: str):
        """Make this be a new identity with the given value.

        Params:
            ident: a string representing the unique identity

        Requires:
            len(ident) > 0

        Effects:
            Initializes this
        """
        assert isinstance(ident, str)
        assert len(ident) > 0
        assert ident is not None

        self.__id = ident

    @property
    def value(self) -> str:
        """Return the value of this identity.

        Returns:
            a string containing the value of this identity

        """
        return self.__id

    def __eq__(self, other) -> bool:
        """Compare this object with another.

        Returns:
            True if the two object are of the same type and value
            False otherwise

        """
        if other is None:
            return False

        if not isinstance(other, Identity):
            return False

        return self.value == other.value

    @staticmethod
    def generate_unique_identifier() -> str:
        """Generate a unique identifier that may be used as part of an identity value.

        Returns:
            A string identity value

        """
        return str(uuid.uuid4())



"""ValueObject is a module providing ValueObject class."""

import abc
from .Identity import Identity


class ValueObject(metaclass=abc.ABCMeta):
    """ValueObject is an immutable class allowing to represent a value.

    All ValueObjects must implement `equals` method.

    """

    def __init__(self, identity: Identity):
        """Make this be a new ValueObject with the given identity.

        Requires:
            idenity have the following pattern : class_name-id

        """
        assert identity is not None
        assert isinstance(identity, Identity)
        assert identity.value.startswith(
            self.__class__.__name__
        ), "This identity is not appropriate for this class"

        self.__identity = identity

    @property
    def identity(self) -> Identity:
        """Return the identity of this entity.

        Returns:
            This entity identity

        """
        return self.__identity

    def __eq__(self, other):
        """Compare self with the given object.

        Returns:
            True if the two objects are equals, False otherwise

        """
        if other is None:
            return False

        if not isinstance(other, ValueObject):
            return False

        if self.identity != other.identity:
            return False

        return self.equals(other)

    @abc.abstractmethod
    def equals(self, other):
        """Compare self with the given object.

        Returns:
            True if the two objects are equals, False otherwise

        """
        raise NotImplementedError(
            f"{self.__class__.__name__} must implement method equals inherited from ValeObjects."
        )

    @classmethod
    def create_identity(cls):
        """Create an identity that is applicable to this entity type.

        The returned identity is formed by this format:
        class_name-random_uuid

        Returns:
            An identity object that is applicable for this object

        """
        ident = Identity.generate_unique_identifier()
        return Identity("{}-{}".format(cls.__name__, ident))
